<?php

/**
 * @file
 * Defines default request handler class
 */

/**
 * Default request handler class for personalized fragments.
 *
 * This class is responsible for handling a request and returning the result
 * along with appropriate HTTP headers back to the client.
 */
class AuthcacheP13nDefaultRequestHandler {
  const PER_USER = 0x0002;
  const PER_PAGE = 0x0004;

  /**
   * @var AuthcacheP13nCoreService
   */
  protected $_coreService;

  /**
   * @var AuthcacheP13nRequestValidator
   */
  protected $_requestValidator;

  /**
   * @var AuthcacheP13nContentBuilder
   */
  protected $_contentBuilder;

  /**
   * @var AuthcacheP13nContentEncoder
   */
  protected $_contentEncoder;

  /**
   * Bootstrap phase required to serve this request.
   */
  protected $_bootstrapPhase;

  /**
   * Number of seconds the response should be cached in the users browser.
   */
  protected $_cacheMaxage;

  /**
   * Set to TRUE when the response is identical for all users.
   */
  protected $_cacheGranularity;

  /**
   * Construct a new instance
   */
  public function __construct(
    AuthcacheP13nCoreService $coreService,
    AuthcacheP13nRequestValidator $requestValidator,
    AuthcacheP13nContentBuilder $contentBuilder,
    AuthcacheP13nContentEncoder $contentEncoder,
    $cacheMaxage,
    $cacheGranularity,
    $bootstrapPhase
  ) {
    $this->_coreService = $coreService;
    $this->_requestValidator = $requestValidator;
    $this->_contentBuilder = $contentBuilder;
    $this->_contentEncoder = $contentEncoder;
    $this->_cacheMaxage = $cacheMaxage;
    $this->_cacheGranularity = $cacheGranularity;
    $this->_bootstrapPhase = $bootstrapPhase;
  }

  public function handle($params) {
    $this->addVaryHeader();

    $input = $this->validate($params);
    $this->bootstrap();
    $result = $this->build($input);
    $this->addCacheControlHeader();
    print $this->encode($result);
  }

  public function defaults() {
    url('', array('prefix' => &$prefix));
    $args = array(
      'q' => empty($prefix) ? '' : $prefix,
    );

    // Add path if per-page caching is required
    if ($this->_cacheGranularity & AuthcacheP13nDefaultRequestHandler::PER_PAGE) {
      $args['q'] .= $_GET['q'];
    }

    return $args;
  }

  /**
   * Add Vary: Cookie header if necessary
   */
  protected function addVaryHeader() {
    // Add Vary: Cookie header if necessary
    if ($this->_cacheGranularity & AuthcacheP13nDefaultRequestHandler::PER_USER) {
      $this->_coreService->drupalAddHttpHeader('Vary', 'Cookie');
    }
  }

  /**
   * Add Cache-Control header
   */
  protected function addCacheControlHeader() {
    if ($this->_cacheMaxage) {
      $this->_coreService->drupalAddHttpHeader('Cache-Control', 'public, max-age=' . $this->_cacheMaxage);
    }
    else {
      $this->_coreService->drupalAddHttpHeader('Cache-Control', 'no-cache, must-revalidate, post-check=0, pre-check=0');
    }
  }

  /**
   * Ensure that drupal bootstrap reached at least the necessary phase for
   * fullfilling the request.
   */
  protected function bootstrap() {
    if ($this->_bootstrapPhase) {
      $this->_coreService->drupalBootstrap($this->_bootstrapPhase);
    }
  }

  /**
   * Return validated input parameters.
   *
   * @throws: AuthcacheP13nRequestInvalidInput
   */
  protected function validate($params) {
    return $this->_requestValidator ? $this->_requestValidator->validate($params) : $params;
  }

  /**
   * Construct response from the validated input
   */
  protected function build($input) {
    return $this->_contentBuilder->build($input);
  }

  /**
   * Encode the result returned by ::build().
   */
  protected function encode($result) {
    $this->_coreService->drupalAddHttpHeader('Content-Type', $this->_contentEncoder->contentType());
    return $this->_contentEncoder->encode($result);
  }
}
